import 'dart:io';

import 'package:dio/adapter.dart';
import 'package:dio/dio.dart';

import '../interceptor/retry.dart';

class DioClient {
  Dio _dio = Dio();

  Dio get $dio => _dio;

  LogInterceptor _logInterceptor;
  RetryInterceptor _retryInterceptor;

  set baseUrl(String url) {
    $dio.options.baseUrl = url;
  }

  set connectTimeout(int timeout) {
    $dio.options.connectTimeout = timeout;
  }

  set receiveTimeout(int timeout) {
    $dio.options.receiveTimeout = timeout;
  }

  set sendTimeout(int timeout) {
    $dio.options.sendTimeout = timeout;
  }

  set contentType(String contentType) {
    $dio.options.contentType = contentType;
  }

  set responseType(ResponseType type) {
    $dio.options.responseType = type;
  }

  set headers(Map<String, dynamic> headers) {
    if (headers != null && headers.isNotEmpty) {
      $dio.options.headers.addAll(headers);
    }
  }

  set queryParameters(Map<String, dynamic> parameters) {
    if (parameters != null && parameters.isNotEmpty) {
      $dio.options.queryParameters.addAll(parameters);
    }
  }

  set extras(Map<String, dynamic> extras) {
    if (extras != null && extras.isNotEmpty) {
      $dio.options.extra.addAll(extras);
    }
  }

  set validateStatus(ValidateStatus validateStatus) {
    $dio.options.validateStatus = validateStatus;
  }

  set receiveDataWhenStatusError(bool receiveDataWhenStatusError) {
    $dio.options.receiveDataWhenStatusError = receiveDataWhenStatusError;
  }

  set followRedirects(bool followRedirects) {
    $dio.options.followRedirects = followRedirects;
  }

  set maxRedirects(int maxRedirects) {
    $dio.options.maxRedirects = maxRedirects;
  }

  set requestEncoder(RequestEncoder requestEncoder) {
    $dio.options.requestEncoder = requestEncoder;
  }

  set responseDecoder(ResponseDecoder responseDecoder) {
    $dio.options.responseDecoder = responseDecoder;
  }

  set proxy(String proxy) {
    ($dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (client) {
      if (proxy != null) {
        client.findProxy = (uri) => "PROXY $proxy";
      }
    };
  }

  set certificate(String pem) {
    ($dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (client) {
      client.badCertificateCallback = (cert, host, port) => cert.pem == pem;
    };
  }

  set certificateFile(File file) {
    ($dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (client) {
      return HttpClient(context: SecurityContext()..setTrustedCertificates(file.path));
    };
  }

  void addInterceptor(Interceptor interceptor) {
    $dio.interceptors.add(interceptor);
  }

  void addInterceptors(List<Interceptor> interceptors) {
    $dio.interceptors.addAll(interceptors);
  }

  void openLog({
    bool request = true,
    bool requestHeader = false,
    bool requestBody = false,
    bool responseHeader = false,
    bool responseBody = false,
  }) {
    closeLog();
    _logInterceptor = LogInterceptor(
        request: request,
        requestHeader: requestHeader,
        requestBody: requestBody,
        responseHeader: responseHeader,
        responseBody: responseBody);
    addInterceptor(_logInterceptor);
  }

  void closeLog() {
    if (_logInterceptor != null) {
      $dio.interceptors.remove(_logInterceptor);
    }
  }

  void openRetry({int retries = 3, Duration interval = const Duration(seconds: 5)}) {
    closeRetry();
    _retryInterceptor = RetryInterceptor($dio, retries, interval);
    addInterceptor(_retryInterceptor);
  }

  void closeRetry() {
    if (_retryInterceptor != null) {
      $dio.interceptors.remove(_retryInterceptor);
    }
  }
}
